# frozen_string_literal: true

module Gitlab
  class Experiment
    module Callbacks
      extend ActiveSupport::Concern
      include ActiveSupport::Callbacks

      included do
        define_callbacks(:unsegmented_run)
        define_callbacks(:segmented_run)
      end

      class_methods do
        def segment(*filter_list, variant:, **options, &block)
          filters = filter_list.unshift(block).compact.map do |filter|
            result_lambda = ActiveSupport::Callbacks::CallTemplate.build(filter, self).make_lambda
            ->(target) { target.variant(variant) if !target.variant_assigned? && result_lambda.call(target, nil) }
          end

          raise ArgumentError, 'no filters provided' if filters.empty?

          set_callback(:segmented_run, :before, *filters, options)
        end
      end
    end
  end
end
